(* Automatically generated code - DO NOT MODIFY *)

open S

type sr_not_attached =
        string
with rpc
exception Sr_not_attached of string
(** An SR must be attached in order to access volumes *)
type sr_does_not_exist =
        string
with rpc
exception SR_does_not_exist of string
(** The specified SR could not be found *)
type volume_does_not_exist =
        string
with rpc
exception Volume_does_not_exist of string
(** The specified volume could not be found in the SR *)
type unimplemented =
        string
with rpc
exception Unimplemented of string
(** The operation has not been implemented *)
type cancelled =
        string
with rpc
exception Cancelled of string
(** The task has been asynchronously cancelled *)
let response_of_exn = function
    | Sr_not_attached(arg_1) -> Rpc.failure (Rpc.Enum [ Rpc.String "Sr_not_attached"; rpc_of_sr_not_attached (arg_1) ])
    | SR_does_not_exist(arg_1) -> Rpc.failure (Rpc.Enum [ Rpc.String "SR_does_not_exist"; rpc_of_sr_does_not_exist (arg_1) ])
    | Volume_does_not_exist(arg_1) -> Rpc.failure (Rpc.Enum [ Rpc.String "Volume_does_not_exist"; rpc_of_volume_does_not_exist (arg_1) ])
    | Unimplemented(arg_1) -> Rpc.failure (Rpc.Enum [ Rpc.String "Unimplemented"; rpc_of_unimplemented (arg_1) ])
    | Cancelled(arg_1) -> Rpc.failure (Rpc.Enum [ Rpc.String "Cancelled"; rpc_of_cancelled (arg_1) ])
    | e -> Rpc.failure (Rpc.Enum [ Rpc.String "Internal_error"; Rpc.String (Printexc.to_string e) ])
let result_of_response = function
    | { Rpc.success = true; contents = t } -> Result.return t
    | { Rpc.success = false; contents = Rpc.Enum [ Rpc.String "Sr_not_attached"; args ] } ->
        let arg_1 = sr_not_attached_of_rpc args in
        Result.fail (Sr_not_attached (arg_1))
    | { Rpc.success = false; contents = Rpc.Enum [ Rpc.String "SR_does_not_exist"; args ] } ->
        let arg_1 = sr_does_not_exist_of_rpc args in
        Result.fail (SR_does_not_exist (arg_1))
    | { Rpc.success = false; contents = Rpc.Enum [ Rpc.String "Volume_does_not_exist"; args ] } ->
        let arg_1 = volume_does_not_exist_of_rpc args in
        Result.fail (Volume_does_not_exist (arg_1))
    | { Rpc.success = false; contents = Rpc.Enum [ Rpc.String "Unimplemented"; args ] } ->
        let arg_1 = unimplemented_of_rpc args in
        Result.fail (Unimplemented (arg_1))
    | { Rpc.success = false; contents = Rpc.Enum [ Rpc.String "Cancelled"; args ] } ->
        let arg_1 = cancelled_of_rpc args in
        Result.fail (Cancelled (arg_1))
    | { Rpc.success = false; contents = t } -> Result.fail (S.Internal_error (Rpc.to_string t))
module Types = struct
    type debug_info =
            string
    with rpc
    (** Debug context from the caller *)
    type key =
            string
    with rpc
    (** Primary key for a volume. This can be any string which is meaningful to the implementation. For example this could be an NFS filename, an LVM LV name or even a URI. This string is abstract. *)
    type sr =
            string
    with rpc
    (** Primary key for a specific Storage Repository. This can be any  string which is meaningful to the implementation. For example this  could be an NFS directory name, an LVM VG name or even a URI. This string is abstract. *)
    type health =
            | Healthy of string (** Storage is fully available *)
            | Recovering of string (** Storage is busy recovering, e.g. rebuilding mirrors. *)
    with rpc
    (** The health status of an SR. *)
    type sr_stat =
            {
            sr: sr; (** The URI identifying this volume. A typical value would be a file:// URI pointing to a directory or block device. *)
            name: string; (** Short, human-readable label for the SR. *)
            description: string; (** Longer, human-readable description of the SR. Descriptions are generally only displayed by clients when the user is examining SRs in detail. *)
            free_space: int64; (** Number of bytes free on the backing storage (in bytes) *)
            total_space: int64; (** Total physical size of the backing storage (in bytes) *)
            datasources: ( string list ); (** URIs naming datasources: time-varying quantities representing anything from disk access latency to free space. The entities named by these URIs are self-describing. *)
            clustered: bool; (** Indicates whether the SR uses clustered local storage. *)
            health: health; (** The health status of the SR. *)
            }
    with rpc
    (** A set of high-level properties associated with an SR. These properties can change dynamically and can be queried by a SR.stat call. *)
    type volume =
            {
            key: key; (** A primary key for this volume. The key must be unique within the enclosing Storage Repository (SR). A typical value would be a filename or an LVM volume name. *)
            uuid: ( string option ); (** A uuid (or guid) for the volume, if one is available. If a storage system has a built-in notion of a guid, then it will be returned here. *)
            name: string; (** Short, human-readable label for the volume. Names are commonly used by when displaying short lists of volumes. *)
            description: string; (** Longer, human-readable description of the volume. Descriptions are generally only displayed by clients when the user is examining volumes individually. *)
            read_write: bool; (** True means the VDI may be written to, false means the volume is read-only. Some storage media is read-only so all volumes are read-only; for example .iso disk images on an NFS share. Some volume are created read-only; for example because they are snapshots of some other VDI. *)
            virtual_size: int64; (** Size of the volume from the perspective of a VM (in bytes) *)
            physical_utilisation: int64; (** Amount of space currently used on the backing storage (in bytes) *)
            uri: ( string list ); (** A list of URIs which can be opened and used for I/O. A URI could  reference a local block device, a remote NFS share, iSCSI LUN or  RBD volume. In cases where the data may be accessed over several  protocols, he list should be sorted into descending order of  desirability. Xapi will open the most desirable URI for which it has  an available datapath plugin. *)
            keys: (( string * string ) list); (** A lit of key=value pairs which have been stored in the Volume  metadata. These should not be interpreted by the Volume plugin. *)
            }
    with rpc
    (** A set of properties associated with a volume. These properties can  change dynamically and can be queried by the Volume.stat call. *)
    type sr_not_attached =
            string
    with rpc
    exception Sr_not_attached of string
    (** An SR must be attached in order to access volumes *)
    type sr_does_not_exist =
            string
    with rpc
    exception SR_does_not_exist of string
    (** The specified SR could not be found *)
    type volume_does_not_exist =
            string
    with rpc
    exception Volume_does_not_exist of string
    (** The specified volume could not be found in the SR *)
    type unimplemented =
            string
    with rpc
    exception Unimplemented of string
    (** The operation has not been implemented *)
    type cancelled =
            string
    with rpc
    exception Cancelled of string
    (** The task has been asynchronously cancelled *)
    module Volume = struct
        module Create = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    sr: 
                            string
                    ;
                    name: 
                            string
                    ;
                    description: 
                            string
                    ;
                    size: 
                            int64
                    ;
                } with rpc
                let make dbg sr name description size = { dbg = dbg; sr = sr; name = name; description = description; size = size }
            end
            module Out = struct
                type t = 
                        volume
                with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[create sr name description size] creates a new volume in [sr] with [name] and [description]. The volume will have size >= [size] i.e. it is always permissable for an implementation to round-up the volume to the nearest convenient block size"
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let sr =
                        let doc = "The Storage Repository" in
                        Arg.(value & opt string "" & info ["sr"] ~doc) in
                    let name =
                        let doc = "A human-readable name to associate with the new disk. This name is intended to be short, to be a good summary of the disk." in
                        Arg.(value & opt string "" & info ["name"] ~doc) in
                    let description =
                        let doc = "A human-readable description to associate with the new disk. This can be arbitrarily long, up to the general string size limit." in
                        Arg.(value & opt string "" & info ["description"] ~doc) in
                    let size =
                        let doc = "A minimum size (in bytes) for the disk. Depending on the characteristics of the implementation this may be rounded up to (for example) the nearest convenient block size. The created disk will not be smaller than this size." in
                        Arg.(value & opt int64 0L & info ["size"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ sr $ name $ description $ size
            end
        end
        module Snapshot = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    sr: 
                            string
                    ;
                    key: 
                            string
                    ;
                } with rpc
                let make dbg sr key = { dbg = dbg; sr = sr; key = key }
            end
            module Out = struct
                type t = 
                        volume
                with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[snapshot sr volume] creates a new volue which is a  snapshot of [volume] in [sr]. Snapshots should never be written to; they are intended for backup/restore only. Note the name and description are copied but any extra metadata associated by [set] is not copied."
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let sr =
                        let doc = "The Storage Repository" in
                        Arg.(value & opt string "" & info ["sr"] ~doc) in
                    let key =
                        let doc = "The volume key" in
                        Arg.(value & opt string "" & info ["key"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ sr $ key
            end
        end
        module Clone = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    sr: 
                            string
                    ;
                    key: 
                            string
                    ;
                } with rpc
                let make dbg sr key = { dbg = dbg; sr = sr; key = key }
            end
            module Out = struct
                type t = 
                        volume
                with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[clone sr volume] creates a new volume which is a writable clone of [volume] in [sr]. Note the name and description are copied but any extra metadata associated by [set] is not copied."
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let sr =
                        let doc = "The Storage Repository" in
                        Arg.(value & opt string "" & info ["sr"] ~doc) in
                    let key =
                        let doc = "The volume key" in
                        Arg.(value & opt string "" & info ["key"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ sr $ key
            end
        end
        module Destroy = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    sr: 
                            string
                    ;
                    key: 
                            string
                    ;
                } with rpc
                let make dbg sr key = { dbg = dbg; sr = sr; key = key }
            end
            module Out = struct
            type t = unit with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[destroy sr volume] removes [volume] from [sr]"
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let sr =
                        let doc = "The Storage Repository" in
                        Arg.(value & opt string "" & info ["sr"] ~doc) in
                    let key =
                        let doc = "The volume key" in
                        Arg.(value & opt string "" & info ["key"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ sr $ key
            end
        end
        module Set_name = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    sr: 
                            string
                    ;
                    key: 
                            string
                    ;
                    new_name: 
                            string
                    ;
                } with rpc
                let make dbg sr key new_name = { dbg = dbg; sr = sr; key = key; new_name = new_name }
            end
            module Out = struct
            type t = unit with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[set_name sr volume new_name] changes the name of [volume]"
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let sr =
                        let doc = "The Storage Repository" in
                        Arg.(value & opt string "" & info ["sr"] ~doc) in
                    let key =
                        let doc = "The volume key" in
                        Arg.(value & opt string "" & info ["key"] ~doc) in
                    let new_name =
                        let doc = "New name" in
                        Arg.(value & opt string "" & info ["new_name"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ sr $ key $ new_name
            end
        end
        module Set_description = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    sr: 
                            string
                    ;
                    key: 
                            string
                    ;
                    new_description: 
                            string
                    ;
                } with rpc
                let make dbg sr key new_description = { dbg = dbg; sr = sr; key = key; new_description = new_description }
            end
            module Out = struct
            type t = unit with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[set_description sr volume new_description] changes the description of [volume]"
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let sr =
                        let doc = "The Storage Repository" in
                        Arg.(value & opt string "" & info ["sr"] ~doc) in
                    let key =
                        let doc = "The volume key" in
                        Arg.(value & opt string "" & info ["key"] ~doc) in
                    let new_description =
                        let doc = "New description" in
                        Arg.(value & opt string "" & info ["new_description"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ sr $ key $ new_description
            end
        end
        module Set = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    sr: 
                            string
                    ;
                    key: 
                            string
                    ;
                    k: 
                            string
                    ;
                    v: 
                            string
                    ;
                } with rpc
                let make dbg sr key k v = { dbg = dbg; sr = sr; key = key; k = k; v = v }
            end
            module Out = struct
            type t = unit with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[set sr volume key value] associates [key] with [value] in the metadata of [volume] Note these keys and values are not interpreted by the plugin; they are intended for the higher-level software only."
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let sr =
                        let doc = "The Storage Repository" in
                        Arg.(value & opt string "" & info ["sr"] ~doc) in
                    let key =
                        let doc = "The volume key" in
                        Arg.(value & opt string "" & info ["key"] ~doc) in
                    let k =
                        let doc = "Key" in
                        Arg.(value & opt string "" & info ["k"] ~doc) in
                    let v =
                        let doc = "Value" in
                        Arg.(value & opt string "" & info ["v"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ sr $ key $ k $ v
            end
        end
        module Unset = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    sr: 
                            string
                    ;
                    key: 
                            string
                    ;
                    k: 
                            string
                    ;
                } with rpc
                let make dbg sr key k = { dbg = dbg; sr = sr; key = key; k = k }
            end
            module Out = struct
            type t = unit with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[unset sr volume key] removes [key] and any value associated with it from the metadata of [volume] Note these keys and values are not interpreted by the plugin; they are intended for the higher-level software only."
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let sr =
                        let doc = "The Storage Repository" in
                        Arg.(value & opt string "" & info ["sr"] ~doc) in
                    let key =
                        let doc = "The volume key" in
                        Arg.(value & opt string "" & info ["key"] ~doc) in
                    let k =
                        let doc = "Key" in
                        Arg.(value & opt string "" & info ["k"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ sr $ key $ k
            end
        end
        module Resize = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    sr: 
                            string
                    ;
                    key: 
                            string
                    ;
                    new_size: 
                            int64
                    ;
                } with rpc
                let make dbg sr key new_size = { dbg = dbg; sr = sr; key = key; new_size = new_size }
            end
            module Out = struct
            type t = unit with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[resize sr volume new_size] enlarges [volume] to be at least [new_size]."
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let sr =
                        let doc = "The Storage Repository" in
                        Arg.(value & opt string "" & info ["sr"] ~doc) in
                    let key =
                        let doc = "The volume key" in
                        Arg.(value & opt string "" & info ["key"] ~doc) in
                    let new_size =
                        let doc = "New disk size" in
                        Arg.(value & opt int64 0L & info ["new_size"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ sr $ key $ new_size
            end
        end
        module Stat = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    sr: 
                            string
                    ;
                    key: 
                            string
                    ;
                } with rpc
                let make dbg sr key = { dbg = dbg; sr = sr; key = key }
            end
            module Out = struct
                type t = 
                        volume
                with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[stat sr volume] returns metadata associated with [volume]."
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let sr =
                        let doc = "The Storage Repository" in
                        Arg.(value & opt string "" & info ["sr"] ~doc) in
                    let key =
                        let doc = "The volume key" in
                        Arg.(value & opt string "" & info ["key"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ sr $ key
            end
        end
        module In = struct
            type t =
                | Create of Create.In.t
                | Snapshot of Snapshot.In.t
                | Clone of Clone.In.t
                | Destroy of Destroy.In.t
                | Set_name of Set_name.In.t
                | Set_description of Set_description.In.t
                | Set of Set.In.t
                | Unset of Unset.In.t
                | Resize of Resize.In.t
                | Stat of Stat.In.t
            with rpc
            let of_call (call: Rpc.call) : (t, 'b) Result.t =
                match call.Rpc.name, call.Rpc.params with
                    | "Volume.create", [ args ] ->
                        Result.return (Create(Create.In.t_of_rpc args))
                    | "Volume.snapshot", [ args ] ->
                        Result.return (Snapshot(Snapshot.In.t_of_rpc args))
                    | "Volume.clone", [ args ] ->
                        Result.return (Clone(Clone.In.t_of_rpc args))
                    | "Volume.destroy", [ args ] ->
                        Result.return (Destroy(Destroy.In.t_of_rpc args))
                    | "Volume.set_name", [ args ] ->
                        Result.return (Set_name(Set_name.In.t_of_rpc args))
                    | "Volume.set_description", [ args ] ->
                        Result.return (Set_description(Set_description.In.t_of_rpc args))
                    | "Volume.set", [ args ] ->
                        Result.return (Set(Set.In.t_of_rpc args))
                    | "Volume.unset", [ args ] ->
                        Result.return (Unset(Unset.In.t_of_rpc args))
                    | "Volume.resize", [ args ] ->
                        Result.return (Resize(Resize.In.t_of_rpc args))
                    | "Volume.stat", [ args ] ->
                        Result.return (Stat(Stat.In.t_of_rpc args))
                    | name, params -> Result.fail (Unknown_method name)
            let call_of = function
                | Create args -> Rpc.call "Volume.create" [ Create.In.rpc_of_t args ]
                | Snapshot args -> Rpc.call "Volume.snapshot" [ Snapshot.In.rpc_of_t args ]
                | Clone args -> Rpc.call "Volume.clone" [ Clone.In.rpc_of_t args ]
                | Destroy args -> Rpc.call "Volume.destroy" [ Destroy.In.rpc_of_t args ]
                | Set_name args -> Rpc.call "Volume.set_name" [ Set_name.In.rpc_of_t args ]
                | Set_description args -> Rpc.call "Volume.set_description" [ Set_description.In.rpc_of_t args ]
                | Set args -> Rpc.call "Volume.set" [ Set.In.rpc_of_t args ]
                | Unset args -> Rpc.call "Volume.unset" [ Unset.In.rpc_of_t args ]
                | Resize args -> Rpc.call "Volume.resize" [ Resize.In.rpc_of_t args ]
                | Stat args -> Rpc.call "Volume.stat" [ Stat.In.rpc_of_t args ]
        end
        module Out = struct
            type t =
                | Create of Create.Out.t
                | Snapshot of Snapshot.Out.t
                | Clone of Clone.Out.t
                | Destroy of Destroy.Out.t
                | Set_name of Set_name.Out.t
                | Set_description of Set_description.Out.t
                | Set of Set.Out.t
                | Unset of Unset.Out.t
                | Resize of Resize.Out.t
                | Stat of Stat.Out.t
            with rpc
            let response_of = function
                | `Error exn ->
                    Rpc.failure (Rpc.String (Printf.sprintf "Internal_error: %s" (Printexc.to_string exn)))
                | `Ok (Create x) ->
                    Rpc.success (Create.Out.rpc_of_t x)
                | `Ok (Snapshot x) ->
                    Rpc.success (Snapshot.Out.rpc_of_t x)
                | `Ok (Clone x) ->
                    Rpc.success (Clone.Out.rpc_of_t x)
                | `Ok (Destroy x) ->
                    Rpc.success (Destroy.Out.rpc_of_t x)
                | `Ok (Set_name x) ->
                    Rpc.success (Set_name.Out.rpc_of_t x)
                | `Ok (Set_description x) ->
                    Rpc.success (Set_description.Out.rpc_of_t x)
                | `Ok (Set x) ->
                    Rpc.success (Set.Out.rpc_of_t x)
                | `Ok (Unset x) ->
                    Rpc.success (Unset.Out.rpc_of_t x)
                | `Ok (Resize x) ->
                    Rpc.success (Resize.Out.rpc_of_t x)
                | `Ok (Stat x) ->
                    Rpc.success (Stat.Out.rpc_of_t x)
        end
    end
    module SR = struct
        module Probe = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    uri: 
                            string
                    ;
                } with rpc
                let make dbg uri = { dbg = dbg; uri = uri }
            end
            module Out = struct
                type t = 
                        {
                        srs: ( sr_stat list ); (** SRs found on this storage device *)
                        uris: ( string list ); (** Other possible URIs which may contain SRs *)
                        }
                with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[probe uri]: looks for existing SRs on the storage device"
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let uri =
                        let doc = "The Storage Repository URI" in
                        Arg.(value & opt string "" & info ["uri"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ uri
            end
        end
        module Create = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    uri: 
                            string
                    ;
                    name: 
                            string
                    ;
                    description: 
                            string
                    ;
                    configuration: 
                            ((
                            string
                            *
                            string
                            ) list)
                    ;
                } with rpc
                let make dbg uri name description configuration = { dbg = dbg; uri = uri; name = name; description = description; configuration = configuration }
            end
            module Out = struct
            type t = unit with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[create uri name description configuration]: creates a fresh SR"
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let uri =
                        let doc = "The Storage Repository URI" in
                        Arg.(value & opt string "" & info ["uri"] ~doc) in
                    let name =
                        let doc = "Human-readable name for the SR" in
                        Arg.(value & opt string "" & info ["name"] ~doc) in
                    let description =
                        let doc = "Human-readable description for the SR" in
                        Arg.(value & opt string "" & info ["description"] ~doc) in
                    let configuration =
                        let doc = "Plugin-specific configuration which describes where and how to create the storage repository. This may include the physical block device name, a remote NFS server and path or an RBD storage pool." in
                        Arg.(value & opt Cmdliner_helpers.string_string_list [] & info ["configuration"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ uri $ name $ description $ configuration
            end
        end
        module Attach = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    uri: 
                            string
                    ;
                } with rpc
                let make dbg uri = { dbg = dbg; uri = uri }
            end
            module Out = struct
                type t = 
                        string
                with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[attach uri]: attaches the SR to the local host. Once an SR is attached then volumes may be manipulated."
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let uri =
                        let doc = "The Storage Repository URI" in
                        Arg.(value & opt string "" & info ["uri"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ uri
            end
        end
        module Detach = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    sr: 
                            string
                    ;
                } with rpc
                let make dbg sr = { dbg = dbg; sr = sr }
            end
            module Out = struct
            type t = unit with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[detach sr]: detaches the SR, clearing up any associated resources. Once the SR is detached then volumes may not be manipulated."
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let sr =
                        let doc = "The Storage Repository" in
                        Arg.(value & opt string "" & info ["sr"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ sr
            end
        end
        module Destroy = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    sr: 
                            string
                    ;
                } with rpc
                let make dbg sr = { dbg = dbg; sr = sr }
            end
            module Out = struct
            type t = unit with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[destroy sr]: destroys the [sr] and deletes any volumes associated with it. Note that an SR must be attached to be destroyed; otherwise Sr_not_attached is thrown."
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let sr =
                        let doc = "The Storage Repository" in
                        Arg.(value & opt string "" & info ["sr"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ sr
            end
        end
        module Stat = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    sr: 
                            string
                    ;
                } with rpc
                let make dbg sr = { dbg = dbg; sr = sr }
            end
            module Out = struct
                type t = 
                        sr_stat
                with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[stat sr] returns summary metadata associated with [sr]. Note this call does not return details of sub-volumes, see SR.ls."
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let sr =
                        let doc = "The Storage Repository" in
                        Arg.(value & opt string "" & info ["sr"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ sr
            end
        end
        module Set_name = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    sr: 
                            string
                    ;
                    new_name: 
                            string
                    ;
                } with rpc
                let make dbg sr new_name = { dbg = dbg; sr = sr; new_name = new_name }
            end
            module Out = struct
            type t = unit with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[set_name sr new_name] changes the name of [sr]"
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let sr =
                        let doc = "The Storage Repository" in
                        Arg.(value & opt string "" & info ["sr"] ~doc) in
                    let new_name =
                        let doc = "The new name of the SR" in
                        Arg.(value & opt string "" & info ["new_name"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ sr $ new_name
            end
        end
        module Set_description = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    sr: 
                            string
                    ;
                    new_description: 
                            string
                    ;
                } with rpc
                let make dbg sr new_description = { dbg = dbg; sr = sr; new_description = new_description }
            end
            module Out = struct
            type t = unit with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[set_description sr new_description] changes the description of [sr]"
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let sr =
                        let doc = "The Storage Repository" in
                        Arg.(value & opt string "" & info ["sr"] ~doc) in
                    let new_description =
                        let doc = "The new description for the SR" in
                        Arg.(value & opt string "" & info ["new_description"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ sr $ new_description
            end
        end
        module Ls = struct
            module In = struct
                type t = {
                    dbg: 
                            debug_info
                    ;
                    sr: 
                            string
                    ;
                } with rpc
                let make dbg sr = { dbg = dbg; sr = sr }
            end
            module Out = struct
                type t = 
                        (
                        volume
                        list
                        )
                with rpc
            end
            module CommandLine = struct
                open Cmdliner
                let doc = "[ls sr] returns a list of volumes contained within an attached SR."
                let t =
                    let dbg =
                        let doc = "Debug context from the caller" in
                        Arg.(value & opt string "" & info ["dbg"] ~doc) in
                    let sr =
                        let doc = "The Storage Repository" in
                        Arg.(value & opt string "" & info ["sr"] ~doc) in
                    let ($) = Term.($) in
                    Term.pure In.make $ dbg $ sr
            end
        end
        module In = struct
            type t =
                | Probe of Probe.In.t
                | Create of Create.In.t
                | Attach of Attach.In.t
                | Detach of Detach.In.t
                | Destroy of Destroy.In.t
                | Stat of Stat.In.t
                | Set_name of Set_name.In.t
                | Set_description of Set_description.In.t
                | Ls of Ls.In.t
            with rpc
            let of_call (call: Rpc.call) : (t, 'b) Result.t =
                match call.Rpc.name, call.Rpc.params with
                    | "SR.probe", [ args ] ->
                        Result.return (Probe(Probe.In.t_of_rpc args))
                    | "SR.create", [ args ] ->
                        Result.return (Create(Create.In.t_of_rpc args))
                    | "SR.attach", [ args ] ->
                        Result.return (Attach(Attach.In.t_of_rpc args))
                    | "SR.detach", [ args ] ->
                        Result.return (Detach(Detach.In.t_of_rpc args))
                    | "SR.destroy", [ args ] ->
                        Result.return (Destroy(Destroy.In.t_of_rpc args))
                    | "SR.stat", [ args ] ->
                        Result.return (Stat(Stat.In.t_of_rpc args))
                    | "SR.set_name", [ args ] ->
                        Result.return (Set_name(Set_name.In.t_of_rpc args))
                    | "SR.set_description", [ args ] ->
                        Result.return (Set_description(Set_description.In.t_of_rpc args))
                    | "SR.ls", [ args ] ->
                        Result.return (Ls(Ls.In.t_of_rpc args))
                    | name, params -> Result.fail (Unknown_method name)
            let call_of = function
                | Probe args -> Rpc.call "SR.probe" [ Probe.In.rpc_of_t args ]
                | Create args -> Rpc.call "SR.create" [ Create.In.rpc_of_t args ]
                | Attach args -> Rpc.call "SR.attach" [ Attach.In.rpc_of_t args ]
                | Detach args -> Rpc.call "SR.detach" [ Detach.In.rpc_of_t args ]
                | Destroy args -> Rpc.call "SR.destroy" [ Destroy.In.rpc_of_t args ]
                | Stat args -> Rpc.call "SR.stat" [ Stat.In.rpc_of_t args ]
                | Set_name args -> Rpc.call "SR.set_name" [ Set_name.In.rpc_of_t args ]
                | Set_description args -> Rpc.call "SR.set_description" [ Set_description.In.rpc_of_t args ]
                | Ls args -> Rpc.call "SR.ls" [ Ls.In.rpc_of_t args ]
        end
        module Out = struct
            type t =
                | Probe of Probe.Out.t
                | Create of Create.Out.t
                | Attach of Attach.Out.t
                | Detach of Detach.Out.t
                | Destroy of Destroy.Out.t
                | Stat of Stat.Out.t
                | Set_name of Set_name.Out.t
                | Set_description of Set_description.Out.t
                | Ls of Ls.Out.t
            with rpc
            let response_of = function
                | `Error exn ->
                    Rpc.failure (Rpc.String (Printf.sprintf "Internal_error: %s" (Printexc.to_string exn)))
                | `Ok (Probe x) ->
                    Rpc.success (Probe.Out.rpc_of_t x)
                | `Ok (Create x) ->
                    Rpc.success (Create.Out.rpc_of_t x)
                | `Ok (Attach x) ->
                    Rpc.success (Attach.Out.rpc_of_t x)
                | `Ok (Detach x) ->
                    Rpc.success (Detach.Out.rpc_of_t x)
                | `Ok (Destroy x) ->
                    Rpc.success (Destroy.Out.rpc_of_t x)
                | `Ok (Stat x) ->
                    Rpc.success (Stat.Out.rpc_of_t x)
                | `Ok (Set_name x) ->
                    Rpc.success (Set_name.Out.rpc_of_t x)
                | `Ok (Set_description x) ->
                    Rpc.success (Set_description.Out.rpc_of_t x)
                | `Ok (Ls x) ->
                    Rpc.success (Ls.Out.rpc_of_t x)
        end
    end
end
module type VOLUME = sig
    include S.M
    val create: Types.Volume.Create.In.t -> (Types.Volume.Create.Out.t, exn) Result.t t
    val snapshot: Types.Volume.Snapshot.In.t -> (Types.Volume.Snapshot.Out.t, exn) Result.t t
    val clone: Types.Volume.Clone.In.t -> (Types.Volume.Clone.Out.t, exn) Result.t t
    val destroy: Types.Volume.Destroy.In.t -> (Types.Volume.Destroy.Out.t, exn) Result.t t
    val set_name: Types.Volume.Set_name.In.t -> (Types.Volume.Set_name.Out.t, exn) Result.t t
    val set_description: Types.Volume.Set_description.In.t -> (Types.Volume.Set_description.Out.t, exn) Result.t t
    val set: Types.Volume.Set.In.t -> (Types.Volume.Set.Out.t, exn) Result.t t
    val unset: Types.Volume.Unset.In.t -> (Types.Volume.Unset.Out.t, exn) Result.t t
    val resize: Types.Volume.Resize.In.t -> (Types.Volume.Resize.Out.t, exn) Result.t t
    val stat: Types.Volume.Stat.In.t -> (Types.Volume.Stat.Out.t, exn) Result.t t
end
module type SR = sig
    include S.M
    val probe: Types.SR.Probe.In.t -> (Types.SR.Probe.Out.t, exn) Result.t t
    val create: Types.SR.Create.In.t -> (Types.SR.Create.Out.t, exn) Result.t t
    val attach: Types.SR.Attach.In.t -> (Types.SR.Attach.Out.t, exn) Result.t t
    val detach: Types.SR.Detach.In.t -> (Types.SR.Detach.Out.t, exn) Result.t t
    val destroy: Types.SR.Destroy.In.t -> (Types.SR.Destroy.Out.t, exn) Result.t t
    val stat: Types.SR.Stat.In.t -> (Types.SR.Stat.Out.t, exn) Result.t t
    val set_name: Types.SR.Set_name.In.t -> (Types.SR.Set_name.Out.t, exn) Result.t t
    val set_description: Types.SR.Set_description.In.t -> (Types.SR.Set_description.Out.t, exn) Result.t t
    val ls: Types.SR.Ls.In.t -> (Types.SR.Ls.Out.t, exn) Result.t t
end
module Volume_server_dispatcher = functor(Impl: VOLUME) -> struct
    type 'a t = 'a Impl.t
    let (>>=) = Impl.(>>=)
    let return = Impl.return
    let dispatch (request: Types.Volume.In.t) : (Types.Volume.Out.t, 'b) Result.t Impl.t = match request with
        | Types.Volume.In.Create x ->
            Impl.create x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.Volume.Out.Create ok)
            ))
        | Types.Volume.In.Snapshot x ->
            Impl.snapshot x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.Volume.Out.Snapshot ok)
            ))
        | Types.Volume.In.Clone x ->
            Impl.clone x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.Volume.Out.Clone ok)
            ))
        | Types.Volume.In.Destroy x ->
            Impl.destroy x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.Volume.Out.Destroy ok)
            ))
        | Types.Volume.In.Set_name x ->
            Impl.set_name x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.Volume.Out.Set_name ok)
            ))
        | Types.Volume.In.Set_description x ->
            Impl.set_description x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.Volume.Out.Set_description ok)
            ))
        | Types.Volume.In.Set x ->
            Impl.set x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.Volume.Out.Set ok)
            ))
        | Types.Volume.In.Unset x ->
            Impl.unset x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.Volume.Out.Unset ok)
            ))
        | Types.Volume.In.Resize x ->
            Impl.resize x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.Volume.Out.Resize ok)
            ))
        | Types.Volume.In.Stat x ->
            Impl.stat x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.Volume.Out.Stat ok)
            ))
    let rpc (call: Rpc.call) : Rpc.response Impl.t =
        match Types.Volume.In.of_call call with
            | `Ok x ->
                dispatch x >>= fun result ->
                return (Types.Volume.Out.response_of result)
            | `Error e ->
                return (response_of_exn e)
end
module Volume_skeleton = functor(M: S.M) -> struct
    include M
    let create x =
        let open Types.Volume.Create in
        let open Types in
        return (`Error (Unimplemented "Volume.create"))
    let snapshot x =
        let open Types.Volume.Snapshot in
        let open Types in
        return (`Error (Unimplemented "Volume.snapshot"))
    let clone x =
        let open Types.Volume.Clone in
        let open Types in
        return (`Error (Unimplemented "Volume.clone"))
    let destroy x =
        let open Types.Volume.Destroy in
        let open Types in
        return (`Error (Unimplemented "Volume.destroy"))
    let set_name x =
        let open Types.Volume.Set_name in
        let open Types in
        return (`Error (Unimplemented "Volume.set_name"))
    let set_description x =
        let open Types.Volume.Set_description in
        let open Types in
        return (`Error (Unimplemented "Volume.set_description"))
    let set x =
        let open Types.Volume.Set in
        let open Types in
        return (`Error (Unimplemented "Volume.set"))
    let unset x =
        let open Types.Volume.Unset in
        let open Types in
        return (`Error (Unimplemented "Volume.unset"))
    let resize x =
        let open Types.Volume.Resize in
        let open Types in
        return (`Error (Unimplemented "Volume.resize"))
    let stat x =
        let open Types.Volume.Stat in
        let open Types in
        return (`Error (Unimplemented "Volume.stat"))
end
module Volume_test = functor(M: S.M) -> struct
    include M
    let create x =
        let open Types.Volume.Create in
        let open Types in
        return (`Ok (Out.({ key = "string"; uuid = Some "string"; name = "string"; description = "string"; read_write = true; virtual_size = 0L; physical_utilisation = 0L; uri = [ "string"; "string" ]; keys = [ ("string", "string") ] })))
    let snapshot x =
        let open Types.Volume.Snapshot in
        let open Types in
        return (`Ok (Out.({ key = "string"; uuid = Some "string"; name = "string"; description = "string"; read_write = true; virtual_size = 0L; physical_utilisation = 0L; uri = [ "string"; "string" ]; keys = [ ("string", "string") ] })))
    let clone x =
        let open Types.Volume.Clone in
        let open Types in
        return (`Ok (Out.({ key = "string"; uuid = Some "string"; name = "string"; description = "string"; read_write = true; virtual_size = 0L; physical_utilisation = 0L; uri = [ "string"; "string" ]; keys = [ ("string", "string") ] })))
    let destroy x =
        let open Types.Volume.Destroy in
        let open Types in
        return (`Ok ())
    let set_name x =
        let open Types.Volume.Set_name in
        let open Types in
        return (`Ok ())
    let set_description x =
        let open Types.Volume.Set_description in
        let open Types in
        return (`Ok ())
    let set x =
        let open Types.Volume.Set in
        let open Types in
        return (`Ok ())
    let unset x =
        let open Types.Volume.Unset in
        let open Types in
        return (`Ok ())
    let resize x =
        let open Types.Volume.Resize in
        let open Types in
        return (`Ok ())
    let stat x =
        let open Types.Volume.Stat in
        let open Types in
        return (`Ok (Out.({ key = "string"; uuid = Some "string"; name = "string"; description = "string"; read_write = true; virtual_size = 0L; physical_utilisation = 0L; uri = [ "string"; "string" ]; keys = [ ("string", "string") ] })))
end
module SR_server_dispatcher = functor(Impl: SR) -> struct
    type 'a t = 'a Impl.t
    let (>>=) = Impl.(>>=)
    let return = Impl.return
    let dispatch (request: Types.SR.In.t) : (Types.SR.Out.t, 'b) Result.t Impl.t = match request with
        | Types.SR.In.Probe x ->
            Impl.probe x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.SR.Out.Probe ok)
            ))
        | Types.SR.In.Create x ->
            Impl.create x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.SR.Out.Create ok)
            ))
        | Types.SR.In.Attach x ->
            Impl.attach x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.SR.Out.Attach ok)
            ))
        | Types.SR.In.Detach x ->
            Impl.detach x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.SR.Out.Detach ok)
            ))
        | Types.SR.In.Destroy x ->
            Impl.destroy x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.SR.Out.Destroy ok)
            ))
        | Types.SR.In.Stat x ->
            Impl.stat x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.SR.Out.Stat ok)
            ))
        | Types.SR.In.Set_name x ->
            Impl.set_name x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.SR.Out.Set_name ok)
            ))
        | Types.SR.In.Set_description x ->
            Impl.set_description x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.SR.Out.Set_description ok)
            ))
        | Types.SR.In.Ls x ->
            Impl.ls x
            >>= fun result ->
            return (Result.(>>=) result (fun ok ->
                Result.return (Types.SR.Out.Ls ok)
            ))
    let rpc (call: Rpc.call) : Rpc.response Impl.t =
        match Types.SR.In.of_call call with
            | `Ok x ->
                dispatch x >>= fun result ->
                return (Types.SR.Out.response_of result)
            | `Error e ->
                return (response_of_exn e)
end
module SR_skeleton = functor(M: S.M) -> struct
    include M
    let probe x =
        let open Types.SR.Probe in
        let open Types in
        return (`Error (Unimplemented "SR.probe"))
    let create x =
        let open Types.SR.Create in
        let open Types in
        return (`Error (Unimplemented "SR.create"))
    let attach x =
        let open Types.SR.Attach in
        let open Types in
        return (`Error (Unimplemented "SR.attach"))
    let detach x =
        let open Types.SR.Detach in
        let open Types in
        return (`Error (Unimplemented "SR.detach"))
    let destroy x =
        let open Types.SR.Destroy in
        let open Types in
        return (`Error (Unimplemented "SR.destroy"))
    let stat x =
        let open Types.SR.Stat in
        let open Types in
        return (`Error (Unimplemented "SR.stat"))
    let set_name x =
        let open Types.SR.Set_name in
        let open Types in
        return (`Error (Unimplemented "SR.set_name"))
    let set_description x =
        let open Types.SR.Set_description in
        let open Types in
        return (`Error (Unimplemented "SR.set_description"))
    let ls x =
        let open Types.SR.Ls in
        let open Types in
        return (`Error (Unimplemented "SR.ls"))
end
module SR_test = functor(M: S.M) -> struct
    include M
    let probe x =
        let open Types.SR.Probe in
        let open Types in
        return (`Ok (Out.({ srs = [ { sr = "string"; name = "string"; description = "string"; free_space = 0L; total_space = 0L; datasources = [ "string"; "string" ]; clustered = true; health = Healthy "string" }; { sr = "string"; name = "string"; description = "string"; free_space = 0L; total_space = 0L; datasources = [ "string"; "string" ]; clustered = true; health = Healthy "string" } ]; uris = [ "string"; "string" ] })))
    let create x =
        let open Types.SR.Create in
        let open Types in
        return (`Ok ())
    let attach x =
        let open Types.SR.Attach in
        let open Types in
        return (`Ok (Out.("string")))
    let detach x =
        let open Types.SR.Detach in
        let open Types in
        return (`Ok ())
    let destroy x =
        let open Types.SR.Destroy in
        let open Types in
        return (`Ok ())
    let stat x =
        let open Types.SR.Stat in
        let open Types in
        return (`Ok (Out.({ sr = "string"; name = "string"; description = "string"; free_space = 0L; total_space = 0L; datasources = [ "string"; "string" ]; clustered = true; health = Healthy "string" })))
    let set_name x =
        let open Types.SR.Set_name in
        let open Types in
        return (`Ok ())
    let set_description x =
        let open Types.SR.Set_description in
        let open Types in
        return (`Ok ())
    let ls x =
        let open Types.SR.Ls in
        let open Types in
        return (`Ok (Out.([ { key = "string"; uuid = Some "string"; name = "string"; description = "string"; read_write = true; virtual_size = 0L; physical_utilisation = 0L; uri = [ "string"; "string" ]; keys = [ ("string", "string") ] }; { key = "string"; uuid = Some "string"; name = "string"; description = "string"; read_write = true; virtual_size = 0L; physical_utilisation = 0L; uri = [ "string"; "string" ]; keys = [ ("string", "string") ] } ])))
end
module Volume_client = functor(RPC: RPC) -> struct
    open RPC
    let create_r x =
        let call = Types.Volume.In.call_of (Types.Volume.In.Create x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.Volume.Create.Out.t_of_rpc x)) in
        return result
    let create ~dbg ~sr ~name ~description ~size =
        let r = Types.Volume.Create.In.({ dbg = dbg; sr = sr; name = name; description = description; size = size }) in
        create_r r
    let snapshot_r x =
        let call = Types.Volume.In.call_of (Types.Volume.In.Snapshot x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.Volume.Snapshot.Out.t_of_rpc x)) in
        return result
    let snapshot ~dbg ~sr ~key =
        let r = Types.Volume.Snapshot.In.({ dbg = dbg; sr = sr; key = key }) in
        snapshot_r r
    let clone_r x =
        let call = Types.Volume.In.call_of (Types.Volume.In.Clone x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.Volume.Clone.Out.t_of_rpc x)) in
        return result
    let clone ~dbg ~sr ~key =
        let r = Types.Volume.Clone.In.({ dbg = dbg; sr = sr; key = key }) in
        clone_r r
    let destroy_r x =
        let call = Types.Volume.In.call_of (Types.Volume.In.Destroy x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.Volume.Destroy.Out.t_of_rpc x)) in
        return result
    let destroy ~dbg ~sr ~key =
        let r = Types.Volume.Destroy.In.({ dbg = dbg; sr = sr; key = key }) in
        destroy_r r
    let set_name_r x =
        let call = Types.Volume.In.call_of (Types.Volume.In.Set_name x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.Volume.Set_name.Out.t_of_rpc x)) in
        return result
    let set_name ~dbg ~sr ~key ~new_name =
        let r = Types.Volume.Set_name.In.({ dbg = dbg; sr = sr; key = key; new_name = new_name }) in
        set_name_r r
    let set_description_r x =
        let call = Types.Volume.In.call_of (Types.Volume.In.Set_description x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.Volume.Set_description.Out.t_of_rpc x)) in
        return result
    let set_description ~dbg ~sr ~key ~new_description =
        let r = Types.Volume.Set_description.In.({ dbg = dbg; sr = sr; key = key; new_description = new_description }) in
        set_description_r r
    let set_r x =
        let call = Types.Volume.In.call_of (Types.Volume.In.Set x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.Volume.Set.Out.t_of_rpc x)) in
        return result
    let set ~dbg ~sr ~key ~k ~v =
        let r = Types.Volume.Set.In.({ dbg = dbg; sr = sr; key = key; k = k; v = v }) in
        set_r r
    let unset_r x =
        let call = Types.Volume.In.call_of (Types.Volume.In.Unset x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.Volume.Unset.Out.t_of_rpc x)) in
        return result
    let unset ~dbg ~sr ~key ~k =
        let r = Types.Volume.Unset.In.({ dbg = dbg; sr = sr; key = key; k = k }) in
        unset_r r
    let resize_r x =
        let call = Types.Volume.In.call_of (Types.Volume.In.Resize x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.Volume.Resize.Out.t_of_rpc x)) in
        return result
    let resize ~dbg ~sr ~key ~new_size =
        let r = Types.Volume.Resize.In.({ dbg = dbg; sr = sr; key = key; new_size = new_size }) in
        resize_r r
    let stat_r x =
        let call = Types.Volume.In.call_of (Types.Volume.In.Stat x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.Volume.Stat.Out.t_of_rpc x)) in
        return result
    let stat ~dbg ~sr ~key =
        let r = Types.Volume.Stat.In.({ dbg = dbg; sr = sr; key = key }) in
        stat_r r
end
module SR_client = functor(RPC: RPC) -> struct
    open RPC
    let probe_r x =
        let call = Types.SR.In.call_of (Types.SR.In.Probe x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.SR.Probe.Out.t_of_rpc x)) in
        return result
    let probe ~dbg ~uri =
        let r = Types.SR.Probe.In.({ dbg = dbg; uri = uri }) in
        probe_r r
    let create_r x =
        let call = Types.SR.In.call_of (Types.SR.In.Create x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.SR.Create.Out.t_of_rpc x)) in
        return result
    let create ~dbg ~uri ~name ~description ~configuration =
        let r = Types.SR.Create.In.({ dbg = dbg; uri = uri; name = name; description = description; configuration = configuration }) in
        create_r r
    let attach_r x =
        let call = Types.SR.In.call_of (Types.SR.In.Attach x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.SR.Attach.Out.t_of_rpc x)) in
        return result
    let attach ~dbg ~uri =
        let r = Types.SR.Attach.In.({ dbg = dbg; uri = uri }) in
        attach_r r
    let detach_r x =
        let call = Types.SR.In.call_of (Types.SR.In.Detach x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.SR.Detach.Out.t_of_rpc x)) in
        return result
    let detach ~dbg ~sr =
        let r = Types.SR.Detach.In.({ dbg = dbg; sr = sr }) in
        detach_r r
    let destroy_r x =
        let call = Types.SR.In.call_of (Types.SR.In.Destroy x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.SR.Destroy.Out.t_of_rpc x)) in
        return result
    let destroy ~dbg ~sr =
        let r = Types.SR.Destroy.In.({ dbg = dbg; sr = sr }) in
        destroy_r r
    let stat_r x =
        let call = Types.SR.In.call_of (Types.SR.In.Stat x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.SR.Stat.Out.t_of_rpc x)) in
        return result
    let stat ~dbg ~sr =
        let r = Types.SR.Stat.In.({ dbg = dbg; sr = sr }) in
        stat_r r
    let set_name_r x =
        let call = Types.SR.In.call_of (Types.SR.In.Set_name x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.SR.Set_name.Out.t_of_rpc x)) in
        return result
    let set_name ~dbg ~sr ~new_name =
        let r = Types.SR.Set_name.In.({ dbg = dbg; sr = sr; new_name = new_name }) in
        set_name_r r
    let set_description_r x =
        let call = Types.SR.In.call_of (Types.SR.In.Set_description x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.SR.Set_description.Out.t_of_rpc x)) in
        return result
    let set_description ~dbg ~sr ~new_description =
        let r = Types.SR.Set_description.In.({ dbg = dbg; sr = sr; new_description = new_description }) in
        set_description_r r
    let ls_r x =
        let call = Types.SR.In.call_of (Types.SR.In.Ls x) in
        RPC.rpc call >>= fun response ->
        let result = Result.(>>=) (result_of_response response) (fun x -> Result.return (Types.SR.Ls.Out.t_of_rpc x)) in
        return result
    let ls ~dbg ~sr =
        let r = Types.SR.Ls.In.({ dbg = dbg; sr = sr }) in
        ls_r r
end